
'------------------------------------------------------------- constructor
PUBLIC CONSTRUCTOR FMOD_()

PRINT_ "- Constructing FMOD object"

'check version
FMOD_has_no_problem = ( FSOUND_GetVersion() >= FMOD_VERSION )
IF FMOD_has_no_problem THEN
   PRINT_ " FMOD version ver." & FSOUND_GetVersion()
ELSE
   PRINT_ " FMOD version error"
END IF

'initialize FMOD with 44100Hz , 32 channels
IF FMOD_has_no_problem THEN this.INIT (44100, 32)

max_SFX = -1
SFX_list_resized = FALSE
load_sfx()

max_BGM = -1
BGM_list_resized = FALSE
load_bgm()

PRINT_ " FMOD is ready"

END CONSTRUCTOR
'------------------------------------------------------------- destructor

PUBLIC DESTRUCTOR FMOD_()

PRINT_ "- Destructing FMOD object"

IF SFX_list_resized THEN
   PRINT_ " freeing BGM list (0.." & max_BGM - 1 & ")"
   FOR i AS INTEGER = 0 TO max_BGM - 1
      FSOUND_Stream_close(BGM_list[i])
   NEXT
   DELETE[] BGM_list
END IF

IF SFX_list_resized THEN
   PRINT_ " freeing SFX list (0.." & max_SFX -1 & ")"
   DELETE[] SFX_list
END IF

IF FMOD_is_initialized THEN
   FSOUND_close
   PRINT_ " FMOD closed"
END IF

END DESTRUCTOR
'-------------------------------------------------------------

SUB FMOD_.PRINT_ (s AS STRING)
   DIM AS INTEGER DEBUG = FREEFILE
   OPEN CONS FOR OUTPUT AS #debug 'debug message log
   ? #debug, s
   CLOSE #debug
END SUB


SUB fmod_.INIT (mixrate AS INTEGER = 44100, maxchannels AS INTEGER = 32)

   FMOD_has_no_problem = FSOUND_Init(mixrate, maxchannels, 0)

   IF FMOD_has_no_problem THEN
      FMOD_is_initialized = TRUE
      PRINT_ " FMOD initialized with " & mixrate & "Hz and " & maxchannels & " channels"
   ELSE
      FMOD_is_initialized = FALSE
      PRINT_ " FMOD init error"
   END IF

END SUB

SUB fmod_.load_SFX(filelist AS STRING = "sfx_list.txt")

   DIM AS INTEGER f
   DIM AS STRING listname

   PRINT_ "- Loading SFX"

   IF FMOD_has_no_problem AND (NOT FMOD_is_disabled) THEN

      'open list and get number and names
      f = FREEFILE
      OPEN "sfx\" + filelist FOR INPUT AS #f
      INPUT #f, max_SFX

      'free previous SFX-list if exist
      IF SFX_list_resized THEN
         PRINT_ " freeing previous SFX-list (0.." & max_SFX - 1 & ")"
         DELETE[] SFX_list
      END IF

      'resize the SFX-list
      PRINT_ " creating SFX-list (0.." & max_SFX - 1 & ")"
      SFX_list = NEW INTEGER PTR [max_SFX - 1]
      SFX_list_resized = TRUE

      'load SFX into the list.
      FOR i AS INTEGER = 0 TO max_SFX - 1
         LINE INPUT #f, listname
         SFX_list[i] = FSOUND_Sample_Load(i, "sfx\" & listname, 0, 0, 0)

         IF SFX_list[i] = 0 THEN
            FMOD_has_no_problem = FALSE
            PRINT_ " SFX load error:" & listname
         else
            PRINT_ " SFX #" & i & " loaded :" & listname
         END IF
      NEXT
      CLOSE #f

   END IF

END SUB

SUB fmod_.load_BGM(filelist AS STRING = "bgm_list.txt")

   DIM AS INTEGER f
   DIM AS STRING listname

   PRINT_ "- Loading BGM"

   IF FMOD_has_no_problem AND (NOT FMOD_is_disabled) THEN

      'open list and get number and names
      f = FREEFILE
      OPEN "bgm\" + filelist FOR INPUT AS #f
      INPUT #f, max_BGM

      'free previous BGM-list if exist
      IF BGM_list_resized THEN
         PRINT_ " freeing previous BGM-list (0.." & max_BGM - 1 & ")"
         FOR i AS INTEGER = 0 TO max_BGM - 1
            FSOUND_Stream_close(BGM_list[i])
         NEXT
         DELETE[] BGM_list
      END IF

      'resize the BGM-list
      PRINT_ " creating BGM-list (0.." & max_BGM - 1 & ")"
      BGM_list = NEW INTEGER PTR [max_BGM - 1]
      BGM_list_resized = TRUE

      'load BGM into the list.
      FOR i AS INTEGER = 0 TO max_BGM - 1
         LINE INPUT #f, listname
         BGM_list[i] = FSOUND_Stream_Open("bgm\" & listname, FSOUND_LOOP_normal, 0, 0)

         IF BGM_list[i] = 0 THEN
            FMOD_has_no_problem = FALSE
            PRINT_ " BGM load error:" & listname
         else
            PRINT_ " BGM #" & i & " loaded :" & listname
         END IF
      NEXT
      CLOSE #f

   END IF

END SUB


SUB fmod_.play_SFX (BYVAL SFX_ID AS INTEGER, BYVAL channel AS INTEGER = FSOUND_FREE, BYVAL volume AS INTEGER = 255)
   'second parameter is sound channel. if omitted, it will use free channel.
   IF FMOD_has_no_problem AND (NOT FMOD_is_disabled) THEN
      IF SFX_ID < max_SFX THEN 'SFX number range check
         FSOUND_Sample_SetDefaults (SFX_list[SFX_ID],  -1 , volume, -1, -1) 'try to change volume
         FSOUND_PlaySound(channel, SFX_list[SFX_ID]) 'play WAV sound on each sound channel
      END IF
   END IF
END SUB

SUB fmod_.play_BGM (BYVAL BGM_ID AS INTEGER, BYVAL channel AS INTEGER = FSOUND_FREE, BYVAL volume AS INTEGER = 255)
   IF FMOD_has_no_problem AND (NOT FMOD_is_disabled) THEN

      IF BGM_ID < max_BGM THEN 'BGM number range check

         'stop previous stream if used
         IF FMOD_bgm_ID THEN FSOUND_Stream_Stop(FMOD_bgm_ID)
         
         'set the id
         FMOD_bgm_ID = BGM_list[BGM_ID]
         
         'reset BGM to the beginning
         FSOUND_Stream_SetTime(FMOD_bgm_ID, 0)   
         
         'play BGM (and get the BGM channel handle to set its volume at the same time)
         FSOUND_SetVolume (FSOUND_Stream_Play(channel, FMOD_bgm_ID), volume)
         
      END if
   END IF
END SUB

SUB fmod_.stop_BGM ()
   IF FMOD_has_no_problem AND (NOT FMOD_is_disabled) THEN
      FSOUND_Stream_Stop(FMOD_bgm_ID)
   END if
END SUB
